import { getCell, getColumnByCell, getRowIdentity } from './util';
import { hasClass, addClass, removeClass,setStyle,on,off } from 'wn-tech-ui/src/utils/dom';
import ElCheckbox from 'wn-tech-ui/packages/checkbox';
import ElTooltip from 'wn-tech-ui/packages/tooltip';
import debounce from 'throttle-debounce/debounce';

export default {
  components: {
    ElCheckbox,
    ElTooltip
  },

  props: {
    store: {
      required: true
    },
    stripe: Boolean,
    shortcutMultipleSelection:Boolean,
    context: {},
    layout: {
      required: true
    },
    rowClassName: [String, Function],
    rowStyle: [Object, Function],
    fixed: String,
    highlight: Boolean
  },

  render(h) {
    const columnsHidden = this.columns.map((column, index) => this.isColumnHidden(index));
    return (
      <table
        class="el-table__body"
        cellspacing="0"
        cellpadding="0"
        border="0">
        <colgroup>
          {
            this._l(this.columns, column =>
              <col
                name={ column.id }
                width={ column.realWidth || column.width }
              />)
          }
        </colgroup>
        <tbody>
          {
            this._l(this.data, (row, $index) =>
              [<tr
                style={ this.rowStyle ? this.getRowStyle(row, $index) : null }
                key={ this.table.rowKey ? this.getKeyOfRow(row, $index) : $index }
                on-dblclick={ ($event) => this.handleDoubleClick($event, row) }
                on-click={ ($event) => this.handleClick($event, row,$index) }
                on-contextmenu={ ($event) => this.handleContextMenu($event, row) }
                on-mouseenter={ _ => this.handleMouseEnter($index) }
                on-mouseleave={ _ => this.handleMouseLeave() }
                class={ [this.getRowClass(row, $index)] }>
                {
                  this._l(this.columns, (column, cellIndex) =>
                    <td
                      class={ [column.id, column.align, column.className || '', columnsHidden[cellIndex] ? 'is-hidden' : '' ] }
                      on-mouseenter={ ($event) => this.handleCellMouseEnter($event, row) }
                      on-mouseleave={ this.handleCellMouseLeave }>
                      {
                        column.renderCell.call(this._renderProxy, h, { row, column, $index, store: this.store, _self: this.context || this.table.$vnode.context }, columnsHidden[cellIndex])
                      }
                    </td>
                  )
                }
                {
                  !this.fixed && this.layout.scrollY && this.layout.gutterWidth ? <td class="gutter" /> : ''
                }
              </tr>,
                this.store.states.expandRows.indexOf(row) > -1
                ? (<tr>
                    <td colspan={ this.columns.length } class="el-table__expanded-cell">
                      { this.table.renderExpanded ? this.table.renderExpanded(h, { row, $index, store: this.store }) : ''}
                    </td>
                  </tr>)
                : ''
              ]
            ).concat(
              this._self.$parent.$slots.append
            ).concat(
              <el-tooltip effect={ this.table.tooltipEffect } placement="top" ref="tooltip" content={ this.tooltipContent }></el-tooltip>
            )
          }
        </tbody>
      </table>
    );
  },

  watch: {
    'store.states.hoverRow'(newVal, oldVal) {
      if (!this.store.states.isComplex) return;
      const el = this.$el;
      if (!el) return;
      const rows = el.querySelectorAll('tbody > tr.el-table__row');
      const oldRow = rows[oldVal];
      const newRow = rows[newVal];
      if (oldRow) {
        removeClass(oldRow, 'hover-row');
      }
      if (newRow) {
        addClass(newRow, 'hover-row');
      }
    },
    'store.states.currentRow'(newVal, oldVal) {
      if (!this.highlight) return;
      const el = this.$el;
      if (!el) return;
      const data = this.store.states.data;
      const rows = el.querySelectorAll('tbody > tr.el-table__row');
      const rowsSelect = el.querySelectorAll('tbody > tr.el-table__row.current-row');
      const oldRow = rows[data.indexOf(oldVal)];
      const newRow = rows[data.indexOf(newVal)];
      if (oldRow) {
        removeClass(oldRow, 'current-row');
      } else if (rowsSelect) {
        [].forEach.call(rowsSelect, row => removeClass(row, 'current-row'));
      }
      if (newRow) {
        addClass(newRow, 'current-row');
      }
    },
  },

  computed: {
    table() {
      return this.$parent;
    },

    

    data() {
      return this.store.states.data;
    },

    columnsCount() {
      return this.store.states.columns.length;
    },

    leftFixedCount() {
      return this.store.states.fixedColumns.length;
    },

    rightFixedCount() {
      return this.store.states.rightFixedColumns.length;
    },

    columns() {
      return this.store.states.columns;
    }
  },

  data() {
    return {
      tooltipContent: '',
      multipleSelection: [],
      minIndex:0,
      maxIndex:0,
      currentRow:{},
    };
  },

  created() {
    this.activateTooltip = debounce(50, tooltip => tooltip.handleShowPopper());
  },

  methods: {
    getKeyOfRow(row, index) {
      const rowKey = this.table.rowKey;
      if (rowKey) {
        return getRowIdentity(row, rowKey);
      }
      return index;
    },

    isColumnHidden(index) {
      if (this.fixed === true || this.fixed === 'left') {
        return index >= this.leftFixedCount;
      } else if (this.fixed === 'right') {
        return index < this.columnsCount - this.rightFixedCount;
      } else {
        return (index < this.leftFixedCount) || (index >= this.columnsCount - this.rightFixedCount);
      }
    },

    getRowStyle(row, index) {
      const rowStyle = this.rowStyle;
      if (typeof rowStyle === 'function') {
        return rowStyle.call(null, row, index);
      }
      return rowStyle;
    },

    getRowClass(row, index) {
      const classes = ['el-table__row'];

      if (this.stripe && index % 2 === 1) {
        classes.push('el-table__row--striped');
      }
      const rowClassName = this.rowClassName;
      if (typeof rowClassName === 'string') {
        classes.push(rowClassName);
      } else if (typeof rowClassName === 'function') {
        classes.push(rowClassName.call(null, row, index) || '');
      }

      return classes.join(' ');
    },

    handleCellMouseEnter(event, row) {
      const table = this.table;
      const cell = getCell(event);

      if (cell) {
        const column = getColumnByCell(table, cell);
        const hoverState = table.hoverState = {cell, column, row};
        table.$emit('cell-mouse-enter', hoverState.row, hoverState.column, hoverState.cell, event);
      }

      // 判断是否text-overflow, 如果是就显示tooltip
      const cellChild = event.target.querySelector('.cell');

      if (hasClass(cellChild, 'el-tooltip') && cellChild.scrollWidth > cellChild.offsetWidth) {
        const tooltip = this.$refs.tooltip;

        this.tooltipContent = cell.innerText;
        tooltip.referenceElm = cell;
        tooltip.$refs.popper && (tooltip.$refs.popper.style.display = 'none');
        tooltip.doDestroy();
        tooltip.setExpectedState(true);
        this.activateTooltip(tooltip);
      }
    },

    handleCellMouseLeave(event) {
      const tooltip = this.$refs.tooltip;
      if (tooltip) {
        tooltip.setExpectedState(false);
        tooltip.handleClosePopper();
      }
      const cell = getCell(event);
      if (!cell) return;

      const oldHoverState = this.table.hoverState;
      this.table.$emit('cell-mouse-leave', oldHoverState.row, oldHoverState.column, oldHoverState.cell, event);
    },

    handleMouseEnter(index) {
      this.store.commit('setHoverRow', index);
    },

    handleMouseLeave() {
      this.store.commit('setHoverRow', null);
    },

    handleContextMenu(event, row) {
      this.handleEvent(event, row, 'contextmenu');
    },

    handleDoubleClick(event, row) {
      this.handleEvent(event, row, 'dblclick');
    },
    /**处理行选择,支持shift多选 chrl多选 */
    handleClick(event, row,index) {
      //支持快捷键选择时逻辑
      if(this.shortcutMultipleSelection){
        const el = this.$el;
        if (!el) return;
        //这个是数据列表
        const data = this.store.states.data;
        //上一次点击的记录
        const oldCurrentRow = this.currentRow;
        let minIndex = data.indexOf(oldCurrentRow);
        //当前点击的索引
        let currentIndex =data.indexOf(row);
        this.currentRow = row;

        let ctrlKey = event.ctrlKey;
        let shiftKey = event.shiftKey;
        //获取所有的数据行元素
        const rows = el.querySelectorAll('tbody > tr.el-table__row');
        const newRow = rows[data.indexOf(row)];
        
        if(shiftKey){
          window.getSelection ? window.getSelection().removeAllRanges() : document.selection.empty();
          // on(el,'selectstart',this.limitSelectWord);
          //先清空
          this.multipleSelection=[];
          //当前点击在上一个之后，那么将较大索引置为当前索引
          if(currentIndex>=this.minIndex){
            this.maxIndex = currentIndex;
          }else{
            //如果当前点击在上次点击之前，则把当前最大值置为上次的最小索引，最小索引置为当前
            this.maxIndex = this.minIndex;
            this.minIndex = currentIndex;
          }
          let arr =[];
          //遍历数据，将最大和最小范围内的数据置为选中状态
          for (let index = 0,len = rows.length; index < len; index++) {
            const element = rows[index];
            if(index<=this.maxIndex&&index>=this.minIndex){
              addClass(element, 'current-row');
              arr.push(data[index]);
            }else{
              removeClass(element, 'current-row');
            }
          }
          this.multipleSelection = arr;
          // off(el,'selectstart',this.limitSelectWord);
          window.getSelection ? window.getSelection().removeAllRanges() : document.selection.empty();
        }else if(ctrlKey){
          // setStyle(el.querySelector('tbody'),"user-select","text");
          let findIndex = this.multipleSelection.indexOf(row);
          let arr =[];
          //如果该条记录已经被选中，则移除，否则增加该条记录为选中
          if(findIndex>=0){
            removeClass(newRow, 'current-row');
            this.multipleSelection.splice(findIndex,1);
          }else{
            addClass(newRow, 'current-row');
            this.multipleSelection.push(row);
          }
        }
        else{
          //单纯的点击，则仅仅选中当前行，并且清除其他选择行
          this.multipleSelection=[];
          this.minIndex=currentIndex;
          this.maxIndex=currentIndex;
          this.multipleSelection.push(row);
          addClass(rows[currentIndex], 'current-row');
          for (let index = 0,len = rows.length; index < len; index++) {
              const element = rows[index];
              if(index==currentIndex){
                //addClass(element, 'current-row');
              }else{
                removeClass(element, 'current-row')
                // this.$refs.multipleTable.toggleRowSelection(data[index], false);
              }
          }
          // setStyle(el.querySelector('tbody'),"user-select","text");
        }
        // off(el,'selectstart',this.limitSelectWord);
        const table = this.table;
        //触发多选事件
        table.$emit('multi-selection-change', this.multipleSelection);
        return;
      }

      this.store.commit('setCurrentRow', row);
      this.handleEvent(event, row, 'click');
    },
    /**限制选择文字 */
    limitSelectWord(event){
      event = window.event||event;
      event.returnValue = false;
    },
    handleEvent(event, row, name) {
      const table = this.table;
      const cell = getCell(event);
      let column;
      if (cell) {
        column = getColumnByCell(table, cell);
        if (column) {
          table.$emit(`cell-${name}`, row, column, cell, event);
        }
      }
      table.$emit(`row-${name}`, row, event, column);
    },

    handleExpandClick(row) {
      this.store.commit('toggleRowExpanded', row);
    }
  }
};
